<?php
/**
 * Plugin Name:         Issues Taxonomy
 * Description:         An editorial taxonomy for helping editors keep track of issues
 * Plugin URI:          https://plugins.seindal.dk/plugins/issues-taxonomy/
 * Update URI:          https://plugins.seindal.dk
 * Author:              René Seindal
 * Author URI:          https://plugins.seindal.dk/
 * Donate link:         https://mypos.com/@historywalks
 * Requires Plugins:    rs-base-plugin
 * License:             GPL v2 or later
 * License URI:         https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:         issues-taxonomy
 * Domain Path:         /languages
 * Requires PHP:        7.4
 * Requires at least:   5.0
 * Version:             1.19
 **/

namespace ReneSeindal;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

require_once( ABSPATH . 'wp-admin/includes/taxonomy.php' );

add_action( 'plugins_loaded', function() {
    class IssuesPlugin extends PluginBase {
        // Needed by base class
        protected $plugin_file = __FILE__;

        use PluginBaseSettings;

        // Defaults doesn't work, here we have 'issue', PluginBase generates 'issues'
        protected $option_page = 'issue-taxonomy';

        // Defaults settings

        protected $taxonomy = 'issue';
        protected $post_types = [ 'post' ];

        /*************************************************************
         *
         * Registers the `issue` taxonomy
         *
         *************************************************************/

        function do_init_action_register_editorial() {
            register_taxonomy( $this->taxonomy, [], [
                'hierarchical'          => false,
                'public'                => ( defined( 'WP_CLI' ) && WP_CLI ),
                'show_in_nav_menus'     => true,
                'show_ui'               => true,
                'show_admin_column'     => true,
                'query_var'             => true,
                'rewrite'               => true,
                'capabilities'          => [
                    'manage_terms' => 'edit_posts',
                    'edit_terms'   => 'edit_posts',
                    'delete_terms' => 'edit_posts',
                    'assign_terms' => 'edit_posts',
                ],
                'labels'                => [
                    'name'                       => __( 'Issues', 'issues-taxonomy' ),
                    'singular_name'              => _x( 'Issue', 'taxonomy general name', 'issues-taxonomy' ),
                    'search_items'               => __( 'Search Issues', 'issues-taxonomy' ),
                    'popular_items'              => __( 'Popular Issues', 'issues-taxonomy' ),
                    'all_items'                  => __( 'All Issues', 'issues-taxonomy' ),
                    'parent_item'                => __( 'Parent Issue', 'issues-taxonomy' ),
                    'parent_item_colon'          => __( 'Parent Issue:', 'issues-taxonomy' ),
                    'edit_item'                  => __( 'Edit Issue', 'issues-taxonomy' ),
                    'update_item'                => __( 'Update Issue', 'issues-taxonomy' ),
                    'view_item'                  => __( 'View Issue', 'issues-taxonomy' ),
                    'add_new_item'               => __( 'Add New Issue', 'issues-taxonomy' ),
                    'new_item_name'              => __( 'New Issue', 'issues-taxonomy' ),
                    'separate_items_with_commas' => __( 'Separate Issues with commas', 'issues-taxonomy' ),
                    'add_or_remove_items'        => __( 'Add or remove Issues', 'issues-taxonomy' ),
                    'choose_from_most_used'      => __( 'Choose from the most used Issues', 'issues-taxonomy' ),
                    'not_found'                  => __( 'No Issues found.', 'issues-taxonomy' ),
                    'no_terms'                   => __( 'No Issues', 'issues-taxonomy' ),
                    'menu_name'                  => __( 'Issues', 'issues-taxonomy' ),
                    'items_list_navigation'      => __( 'Issues list navigation', 'issues-taxonomy' ),
                    'items_list'                 => __( 'Issues list', 'issues-taxonomy' ),
                    'most_used'                  => _x( 'Most Used', 'issue', 'issues-taxonomy' ),
                    'back_to_items'              => __( '&larr; Back to Issues', 'issues-taxonomy' ),
                ],
                'show_in_rest'          => true,
                'rest_base'             => $this->taxonomy,
                'rest_controller_class' => 'WP_REST_Terms_Controller',

                'update_count_callback' => '_update_generic_term_count',
            ] );

        }
        /*
         *
         * Sets the post updated messages for the `issue` taxonomy.
         *
         * @param  array $messages Post updated messages.
         * @return array Messages for the `issue` taxonomy.
         *
         */

        function do_term_updated_messages_action_issue_updated_messages( $messages ) {

            $messages[ $this->taxonomy ] = [
                0 => '', // Unused. Messages start at index 1.
                1 => __( 'Issue added.', 'issues-taxonomy' ),
                2 => __( 'Issue deleted.', 'issues-taxonomy' ),
                3 => __( 'Issue updated.', 'issues-taxonomy' ),
                4 => __( 'Issue not added.', 'issues-taxonomy' ),
                5 => __( 'Issue not updated.', 'issues-taxonomy' ),
                6 => __( 'Issues deleted.', 'issues-taxonomy' ),
            ];

            return $messages;
        }

        /*************************************************************
         *
         * Connect taxonomy and post types
         *
         *************************************************************/

        function do_wp_loaded_action() {
            $post_types = $this->get_option( 'post_types', $this->post_types );

            foreach ( $post_types as $post_type )
                if ( get_post_type_object( $post_type ) )
                    register_taxonomy_for_object_type( $this->taxonomy, $post_type );
        }

        /*************************************************************
         *
         * Add filter on post lists on this taxonomy
         *
         *************************************************************/

        function do_restrict_manage_posts_action_issue( $post_type, $which ) {
            if ( ! $this->get_option( 'post_list_filter', false ) )
                return;

            $tax = get_taxonomy( $this->taxonomy );

            if ( in_array( $post_type, $tax->object_type ) ) {

                $selected = '';
                if ( isset( $_REQUEST[ $this->taxonomy ] ) ) {
                    $selected = $_REQUEST[ $this->taxonomy ];
                }

                wp_dropdown_categories(array(
                    'show_option_all' =>  $tax->labels->all_items,
                    'taxonomy'        =>  $this->taxonomy,
                    'value_field'     => 'slug',
                    'name'            =>  $this->taxonomy,
                    'orderby'         =>  'name',
                    'selected'        =>  $selected,
                    'hierarchical'    =>  false,
                    'show_count'      =>  true,
                    'hide_empty'      =>  false,
                ) );
            }
        }


        /*************************************************************
         *
         * Dashboard widget
         *
         *************************************************************/

        function do_wp_dashboard_setup_action_status_widget() {
            wp_add_dashboard_widget(
                'issues_dashboard_widget',
                esc_html__( 'Issue status', 'issues-taxonomy' ),
                [ $this, 'dashboard_widget_render' ]
            );
        }

        /**
         * Create the function to output the content of our Dashboard Widget.
         */
        function dashboard_widget_render() {
            // Get all terms from our taxonomy
            $tax = get_taxonomy( $this->taxonomy );
            $post_types = $tax->object_type;
            $terms = get_terms( [ 'taxonomy' => $this->taxonomy, 'fields' => 'all', ] );

            // Total number of posts for each post type
            $totals = [];
            foreach ( $post_types as $post_type ) {
                $totals[ $post_type ] = count( get_posts( [
                    'post_type' => $post_type,
                    'post_status' => 'any',
                    'numberposts' => -1,
                    'fields' => 'ids',
                ] ) );
            }

            // Count of issues for each term and post type
            $counts = [];
            foreach ( $terms as $term ) {
                foreach ( $post_types as $post_type ) {
                    $counts[$term->term_id][$post_type] = $this->get_post_count_for_term_and_post_type( $term->term_id, $post_type );
                }
            }

            // Remove post types without issues
            $no_issues = array_filter(
                $post_types,
                fn($post_type) => 0 == array_sum( array_map( fn($c) => $c[$post_type], $counts ) )
            );

            $post_types = array_filter(
                $post_types,
                fn($post_type) => 0 < array_sum( array_map( fn($c) => $c[$post_type], $counts ) )
            );

            // Print the table

            $show_percent = $this->get_option( 'percent_cols', false );

            if ( $post_types ) {
                printf( '<table width="100%%">' );

                printf( '<tr>' );
                printf("<th>Issue</th>" );
                foreach ( $post_types as $post_type ) {
                    $post_type_object = get_post_type_object( $post_type );
                    printf("<th align=right>%s</th>", esc_html( $post_type_object->labels->name ) );
                    if ( $show_percent )
                        printf("<th align=right>%%</th>" );
                }
                printf( '</tr>' );

                foreach ( $terms as $term ) {
                    printf( '<tr>' );

                    $url = apply_filters( 'issues_taxonomy_row_link', NULL, $term );
                    if ( empty( $url ) )
                        printf( '<td title="%s">%s</td>', esc_attr( $term->description ), esc_html( $term->name) );
                    else
                        printf( '<td title="%s"><a href="%s">%s</a></td>', esc_attr( $term->description ), esc_url( $url ), esc_html( $term->name) );

                    foreach ( $post_types as $post_type ) {
                        $count = $counts[$term->term_id][$post_type];
                        if ( $count ) {
                            $url = $this->build_admin_url( 'edit.php', [
                                'issue' => $term->slug,
                                'post_type' => $post_type,
                            ] );
                            printf( "<td align=right><a href='%s' class='button-link'>%d</a></td>", esc_url( $url ), $count );
                            if ( $show_percent )
                                printf( "<td align=right>%d%%</td>", 100 * ( $count / $totals[ $post_type] ) );
                        } else {
                            printf( "<td align=right>-</td>" );
                            if ( $show_percent )
                                printf( "<td align=right></td>" );
                        }
                    }
                    printf( '</tr>' );
                }

                printf( '</table>' );
            }

            if ( $no_issues ) {
                printf( '<p>%s: %s</p>',
                        esc_html__( 'These post types have no issues', 'issues-taxonomy' ),
                        join( ', ', array_map( fn($post_type) =>  get_post_type_object( $post_type )->labels->name, $no_issues ) )
                );
            }
        }

        // CSS for the dashboard widget
        function do_admin_head_action_dashboard_css() {
            $screen = get_current_screen();

            if ( isset( $screen ) && 'dashboard' == $screen->base ) {
                echo '<style type="text/css">';
                echo "#issues_dashboard_widget table { width: 100%; }";
                echo "#issues_dashboard_widget table tr:first-child th { border-bottom: 1px solid silver; }";
                echo "#issues_dashboard_widget table tr:nth-child(even) { background: #f6f7f7; }";
                echo '</style>';
            }
        }

        protected function get_post_count_for_term_and_post_type( $term_id, $post_type ) {
            return count ( get_posts( [
                'post_type' => $post_type,
                'post_status' => 'any',
                'numberposts' => -1,
                'tax_query' => [
                    [
                        'taxonomy' => $this->taxonomy,
                        'field' => 'id',
                        'terms' => $term_id,
                    ]
                ],
                'fields' => 'ids',
            ] )
            );
        }

        /************************************************************************
         *
         *	Settings
         *
         ************************************************************************/

        function do_admin_menu_action() {
            $this->settings_add_menu(
                __( 'Issues Taxonomy', 'issues-taxonomy'),
                __( 'Issues', 'issues-taxonomy' )
            );
        }

        // Called automatically by PluginBaseSettings
        function settings_define_sections_and_fields() {
            $section = $this->settings_add_section(
                'section_post_types',
                __( 'Post-types', 'issues-taxonomy' ),
                __( 'Select the post-types you want to issues taxonomy to apply to.', 'issues-taxonomy' )
            );

            $this->settings_build_post_types_menu(
                'post_types', $section,
                __( 'Post-types', 'issues-taxonomy' ),
                [ 'public' => true ],
                $this->post_types,
                'issues_taxonomy_post_types'
            );

            $section = $this->settings_add_section(
                'section_dashboard_widget',
                __( 'Dashboard widget', 'issues-taxonomy' ),
                __( 'Settings for the Issues Taxonomy dashboard widget', 'issues-taxonomy' )
            );

            $this->settings_add_field(
                'percent_cols', $section,
                __( 'Show percent columns', 'issues-taxonomy' ),
                'settings_field_checkbox_html'
            );

            $section = $this->settings_add_section(
                'section_integration',
                __( 'Integrations', 'issues-taxonomy' ),
                __( 'Select how this plugin integrates into the WordPress admin interface', 'issues-taxonomy' )
            );

            $this->settings_add_field(
                'post_list_filter', $section,
                __( 'Allow post filtering', 'issues-taxonomy' ),
                'settings_field_checkbox_html'
            );
        }
    }

    IssuesPlugin::instance();
} );
